import sys,os
import logging
import cgi ,time, Cookie
#import cgitb; cgitb.enable()
import StringIO
import md5
import glob
from simpletal import simpleTAL, simpleTALES
from simpletal.simpleTALUtils import FastStringOutput

if (os.path.join(os.environ['MOBYLEHOME'],'Src')) not in sys.path:
  sys.path.append(os.path.join(os.environ['MOBYLEHOME'],'Src'))
import Mobyle.MobyleLogger
Mobyle.MobyleLogger.MLogger()

import Mobyle.ConfigManager 
import Mobyle.MobyleJob 
import Mobyle.Parser 
from Mobyle.MobyleError import *
from Mobyle.Session import *
import Mobyle.Net

_extra_epydoc_fields__ = [('call', 'Called by','Called by')]
_interval = 10

class MbCGI(object):
    """
    This class defines the basic structure of a Mobyle CGI, that should be reused by every CGI in the Portal
    """

    response = {}
    
    request = cgi.FieldStorage()

    c_log = logging.getLogger('mobyle.cgi')
  
    def __init__(self):
        self.osDir = os.getcwd()
        self.cfg = Mobyle.ConfigManager.Config()

    def process(self):
        raise NotImplementedError      

    def render(self):
        raise NotImplementedError

class TALCGI(MbCGI):

    template_file = ""
    """name of the template file used for the cache"""
  
    def render(self):
        os.chdir(self.osDir)
        self.context = simpleTALES.Context(allowPythonPath=1)    
        templateFile = open(self.template_file)
        self.template = simpleTAL.compileHTMLTemplate (templateFile)
        templateFile.close()
        self.context.addGlobal ("sitemacros", self.template)
        self.context.addGlobal("options", self.response)
        self.context.addGlobal("self", self)
        html = StringIO.StringIO()
        self.template.expand (self.context, html)
        print "Content-type: text/html\n\n" + html.getvalue()

class XSLCGI(TALCGI):

    xslParams = None
    """parameters used when calling an XSL file for rendering"""
  
    def render(self):
        if hasattr(self,'error_msg'):
          self.template_file="error.html"
          super(XSLCGI, self).render()
        else:          
          from Ft.Xml.Xslt.Processor import Processor
          from Ft.Xml.InputSource import DefaultFactory
          xsltproc = Processor()
          xsltproc.appendStylesheet(DefaultFactory.fromUri(self.xsltUri))
          html = xsltproc.run(DefaultFactory.fromUri(self.xmlUrl), topLevelParams=self.xslParams)
          print "Content-type: text/html\n\n" + html

class TextCGI(MbCGI):

    text = ""

    def render(self):
        print "Content-type: text/plain\n\n" + self.text

class JSONCGI(TextCGI):
    
    jsonMap = {}
  
    def render(self):
        import simplejson
        import traceback, StringIO
        if hasattr(self,'error_msg'):
          self.jsonMap['msg']=self.error_msg
        try:
            print "Content-type: text/plain\n\n" + simplejson.dumps(self.jsonMap, encoding='ascii')
        except UnicodeDecodeError, e:
            try:
                print "Content-type: text/plain\n\n" + simplejson.dumps(self.jsonMap, encoding='utf-8')
            except UnicodeDecodeError, e2:
                try:
                    print "Content-type: text/plain\n\n" + simplejson.dumps(self.jsonMap, encoding='iso-8859-1')
                except UnicodeDecodeError, e3:
                    self.c_log.error("error during json content encoding")

class PNGCGI(MbCGI):

    def render(self):
        print "Content-type: image/png\n\n" + self.pngImage
        
class SessionCGI:

    def __init__(self,cgi):
        self.cgi = cgi
        self.c_log = self.cgi.c_log

    def process(self):
        self.load_session()
        self.cgi.process()
        
    def render(self):
        self.print_session()
        self.cgi.render()
        
    def load_session(self):
        self.cgi.resetPortal = False
        # SESSION LOAD
        self.cgi.sessionFactory = SessionFactory(self.cgi.cfg)
        c = Cookie.SimpleCookie(os.environ.get("HTTP_COOKIE"))
        if c.get("sessionKey"):
            self.cgi.sessionKey = c.get("sessionKey").value
        if (hasattr(self.cgi,'sessionKey') and self.cgi.sessionKey):
          try:
              if(c.get("authenticated") and c.get("authenticated").value=="True"):
                  email = Mobyle.Net.Email(c.get("email").value,self.cgi.cfg)
                  self.cgi.session = self.cgi.sessionFactory.getAuthenticatedSession(email,c.get("password").value)            
              else:
                  self.cgi.session = self.cgi.sessionFactory.getAnonymousSession(self.cgi.sessionKey)
          except MobyleError, errMsg:
              # if the session no longer exists, set session to None (should trigger a reload on the client side when reading the cookie)
              self.cgi.session = None
              self.cgi.sessionKey = None
              self.cgi.resetPortal = True
        else:
          # if no session is defined, create one
          if (self.cgi.cfg.anonymousSession()!='no'):
              self.cgi.session = self.cgi.sessionFactory.getAnonymousSession()  
              self.cgi.sessionKey = self.cgi.session.getKey()
          else:
              self.cgi.session = None
              self.cgi.sessionKey = None
        self.read_session()
    
    def print_session(self):
        self.cgi.response['email'] = self.cgi.email
        print self.__getSessionCookieSetString__()
    
    def read_session(self):
        if self.cgi.session:
            self.cgi.email, self.cgi.authenticated, self.cgi.activated = self.cgi.session.getBaseInfo()
        else:
            self.cgi.email, self.cgi.authenticated, self.cgi.activated = (None, False, False)      
        return

    def __getSessionCookieSetString__(self):
        """
        I{__getSessionCookieSetString__} returns the mobyle session cookie string
        """
        c = Cookie.SimpleCookie( )
        c["resetPortal"]=str(self.cgi.resetPortal)
        if self.cgi.session:
            c["sessionKey"]=str(self.cgi.sessionKey)
            c["activated"]=str(self.cgi.activated)
            c["email"]=str(self.cgi.email)
            c["authenticated"]=str(self.cgi.authenticated)
            if(self.cgi.request.getfirst('password')):
                c["password"]=self.cgi.request.getfirst('password')
        else:
            c["sessionKey"]=''
            c["activated"]=''
            c["email"]=''
            c["authenticated"]=''
        return c.output()